# Zavíračka

Maďarsko je stát s $N$ městy číslovány od $0$ do $N-1$.

Města jsou spojena $N-1$ *obousměrnými* silnicemi číslovanými od $0$ do $N-2$.
Pro každé přípustné $j$ silnice $j$ spojuje města $U[j]$ a $V[j]$ a má délku $W[j]$, tedy na její přejetí je třeba $W[j]$ času.

Každá silnice spojuje dvě různá města a každá dvojice měst je spojena nejvýše jednou silnicí.

**Cesta** mezi dvěma různými městy $a$ a $b$ je sekvence různých měst $p_0, p_1, \ldots, p_t$ tak že:
 * $p_0 = a$,
 * $p_t = b$,
 * pro každé přípustné $i$, města $p_i$ a $p_{i + 1}$ jsou spojená silnicí.

Slibujeme, že je možné cestovat mezi libovolnou dvojicí měst.
Z toho je možné dokázat, že pro každou dvojici měst je nejvýše jedna cesta.

**Délka** cesty $p_0, p_1, \ldots, p_t$ je součet délek všech $t$ silnic na cestě.

V Maďarsku by chtělo mnoho lidí cestovat na Hroší festival do jednoho z dvou významných měst, kde se festival odehrává.
Jakmile oslavy Hroších bohů skončí, rádi by se vrátili do svých domovů.
Vláda by ráda zabránila KSPákům (tedy těm, co slaví) narušovat večerní klid místních na jejich návratové cestě,
tedy plánuji v určité časy uzavírat města.
Každému městu vláda přiřadí nezáporný **zavírací čas**.
Dle nařízení ministra součet všech zavíracích časů nesmí přesáhnout $K$.
Přesněji řečeno: Pro každé město $i$ označme jeho nezáporný zavírací čas $c[i]$.
Suma všech $c[i]$ nesmí být větší než $K$.

Uvažme město $a$ a nějaké přiřazení zavíracích časů.
Řekneme, že město $b$ je **dosažitelné** z města $a$ právě když buď $a=b$ nebo cesta $p_0, \ldots, p_t$ mezi nimi splňuje všechny následující podmínky:
 * délka cesty $p_0, p_1$ je nejvýše $c[p_1]$
 * délka cesty $p_0, p_1, p_2$ je nejvýše $c[p_2]$
 * $\ldots$
 * délka cesty $p_0, p_1, p_2, \ldots, p_t$ je nejvýše $c[p_t]$.

Letos se bude Hroší festival odehrávat ve dvou městech $X$ a $Y$.
Pro každé přiřazení zavíracích časů definujeme **hroší skóre**
jako součet následujících čísel:
- Počet měst dosažitelných z města $X$.
- Počet měst dosažitelných z města $Y$.

Upozorňujeme, že kdy je město dosažitelné z $X$ i z $Y$, počítá se *dvakrát*!

Vaším úkolem je spočítat maximální možné hroší skóre, kterého je možné dosáhnout vhodným přiřazením zavíracích časů.


## Implementační detaily

Napište následující funkci:

```
int max_score(int N, int X, int Y, int64 K, int[] U, int[] V, int[] W)
```

* $N$: počet měst
* $X$, $Y$: města, kde se budou odehrávat hroší festivaly.
* $K$: horní limit na součet zavíracích časů.
* $U$, $V$: pole délky $N - 1$ popisující cesty.
* $W$: pole délky $N - 1$ udávající délky cest.
* Funkce má vrátit maximáľní možné hroší skóre kterého lze dosáhnout nějakým přiřazením zavíracích časů.
* Tato funkce může být zavolána **několikrát** během jednoho spuštění programu (test casu).

## Příklady

Uvažme následující zavolání:

```
max_score(7, 0, 2, 10,
          [0, 0, 1, 2, 2, 5], [1, 3, 2, 4, 5, 6], [2, 3, 4, 2, 5, 3])
```

To odpovídá následujícímu plánku:

![](closing-sample1.png)

Když přiřadíme zavírací časy následovně:

| **City**         | $0$ | $1$ | $2$ | $3$ | $4$ | $5$ | $6$ |
|:----------------:|:---:|:---:|:---:|:---:|:---:|:---:|:---:|
| **Closing time** | $0$ | $4$ | $0$ | $3$ | $2$ | $0$ | $0$ |


Suma časů je $9 \le 10$, takže OK.
Města $0$, $1$ a $3$ jsou dostupná z $X = 0$ a $1$, $2$ a $4$ jsou dostupná z $Y=2$.
Tedy hroší skóre je $3 + 3 = 6$.
Líp to nejde, takže správná odpověď je $6$.

Dále uvažme následující volání:

```
max_score(4, 0, 3, 20, [0, 1, 2], [1, 2, 3], [18, 1, 19])
```

To odpovídá:

![](closing-sample2.png)

Když nastavíme časy:

| **City**         | $0$ | $1$ | $2$ | $3$ |
|:----------------:|:---:|:---:|:---:|:---:|
| **Closing time** | $0$ | $1$ | $19$| $0$ |

Město $0$ je dostupné z $X = 0$
a města $2$ a $3$ are jsou dostupné z $Y = 3$.
Proto hroší skóre je $1+2=3$.
I zde je to optimální, takže odpověď je $3$.

## Limity

* $2 \le N \le 200\,000$
* $0 \le X \lt Y \lt N$
* $0 \le K \le 10^{18}$
* $0 \le U[j] \lt V[j] \lt N$ (pro každé přípustné $j$)
* $1 \le W[j] \le 10^6$ (pro každé přípustné $j$)
* Je možné se z každého města dostat do každého jiného.
* $S_N \le 200\,000$, kde $S_N$ je součet $N$ přes všechna volání `max_score` v jednom spuštění programu (test casu).


## Podúlohy

Síť cest je **lineární** když cesta $i$ spojuje města $i$ and $i+1$
pro každé přípustné $i$.

1. (8 bodů) Délka cesty z $X$ do $Y$ je větší než $2K$.
1. (9 bodů) $S_N \le 50$, síť cest je lineární.
1. (12 bodů) $S_N \le 500$, síť cest je lineární.
1. (14 bodů) $S_N \le 3\,000$, síť cest je lineární.
1. (9 bodů) $S_N \le 20$
1. (11 bodů) $S_N \le 100$
1. (10 bodů) $S_N \le 500$
1. (10 bodů) $S_N \le 3\,000$
1. (17 bodů) Bez dalších omezení


## Ukázkový Grader

Nechť $C$ je počet scénářů, tedy počet volání  `max_score` v jednom spuštění programu.
Grader čte data v tomto formátu:

* řádek $1$: $C$

Následuje popis $C$ scénářů, každý z nich v následujícím formátu:

* řádek $1$: $N \; X \; Y \; K$
* řádek $2 + j$ ($0 \le j \le N - 2$): $U[j] \; V[j] \; W[j]$

Pro každý scénář vypíše grader jeden řádek:

* řádek $1$: návratová hodnota `max_score`
