# Zárvatartási idő (Closing Time)

Magyarországon $N$ város van, $0$-tól $N-1$-ig számozva.

A városokat  $N - 1$ darab *kétirányú* út köti össze, amiket $0$-tól $N - 2$-ig számoztak meg.
Minden  $j$ esetén ( $0 \le j \le N - 2$ ) a   $j$. út az $U[j]$. és a $V[j]$. várost köti össze és hossza $W[j]$,
azaz a városok között $W[j]$ időegység alatt lehet egy-egy alkalommal utazni.
Minden út két különböző várost köt össze, és minden különböző várospárt legfeljebb egy út köt össze.

Két különböző $a$ és $b$ város közötti **útvonal** olyan különböző városok $p_0, p_1, \ldots, p_t$ sorozata, ahol:
 * $p_0 = a$,
 * $p_t = b$,
 * és minden $i$ ($0 \le i \lt t$) esetén létezik út, amely összeköti a $p_i$. és $p_{i + 1}$. városokat.

Az utak segítségével bármelyik városból bármelyik másik városba el lehet jutni, azaz bármely két különböző város között létezik útvonal.
Bizonyítható, hogy ez az útvonal minden különböző várospár esetében egyedi.

Egy $p_0, p_1, \ldots, p_t$ útvonal **hossza** az útvonal mentén az egymást követő városokat összekötő $t$ darab ut hosszainak összege.

Magyarországon az Államalapítás ünnepére sokan utaznak két nagyvárosba.
Az ünnepségek után a lakosok visszatérnek otthonaikba.
A kormány meg akarja akadályozni, hogy a tömeg zavarja a helyieket, ezért azt tervezik, hogy bizonyos időpontokban lezárják a városokat.
A kormány minden egyes városnak megad egy nem-negatív **zárvatartási időt**.
A kormány úgy döntött, hogy az összes zárvatartási idő összege nem lehet több, mint $K$.
Pontosabban, minden $i$-re ( $0 \le i \lt N - 1$) a zárvatartási idő egy nemnegatív egész szám, $c[i]$.
Az összes $c[i]$ összege nem lehet nagyobb, mint $K$.

Nézzünk egy $a$ várost és a zárvatartási idők valamilyen hozzárendelését.
Azt mondjuk, hogy egy $b$ város **elérhető** az $a$ városból, akkor és csak akkor, ha vagy $b = a$, vagy létezik a két város közötti olyan $p_0, \ldots, p_t$ útvonal (amiben $p_0 = a$ és $p_t = b$), ami kielégíti a következő feltételeket:
 * a $p_0, p_1$ útvonal hossza legfeljebb $c[p_1]$, és
 * a $p_0, p_1, p_2$ út hossza legfeljebb $c[p_2]$, és
 * $\ldots$
 * a $p_0, p_1, p_2, \ldots, p_t$ útvonal hossza legfeljebb $c[p_t]$.

Idén a két fő fesztiválhelyszín az $X$ és az $Y$ városban található.
A zárvatartási idők minden egyes hozzárendelésénél a **kényelmi pontszám** a következő két szám összege:
- Az $X$ városból elérhető városok száma.
- Az $Y$ városból elérhető városok száma.

Megjegyzés: ha egy város elérhető $X$ városból és elérhető $Y$ városból is, akkor ez a város *kétszeresen* számít a kényelmi pontszámba.

A feladatod a maximális kényelmi pontszám kiszámítása, amely a zárvatartási idők valamilyen hozzárendelésével elérhető.

## Megvalósítás

A következő függvényt kell megvalósítanod:

```
int max_score(int N, int X, int Y, int64 K, int[] U, int[] V, int[] W)
```

* $N$: a városok száma.
* $X$, $Y$: a két fesztiválváros.
* $K$: a zárvatartási idők összegének felső határa.
* $U$, $V$:  $N - 1$ hosszúságú tömbök, amik leírják a városok közti kapcsolatokat.
* $W$: $N - 1$ hosszúságú tömb, ami tartalmazza az utak hosszát.
* A függvény visszatérési értéke legyen a maximális kényelmi pontszám, ami elérhető valamely zárvatartási idő hozzárendeléssel.
* Ez a függvény minden egyes tesztesetre **több alkalommal** is meghívható .

## Példa

Nézzük a következő függvényhívást:

```
max_score(7, 0, 2, 10,
          [0, 0, 1, 2, 2, 5], [1, 3, 2, 4, 5, 6], [2, 3, 4, 2, 5, 3])
```

Ez az alábbi úthálózatot írja le:

![](closing-sample1.png)


Tegyük fel, hogy a zárvatartási időket a következőképpen osztják ki:


| **Város**         | $0$ | $1$ | $2$ | $3$ | $4$ | $5$ | $6$ |
|:----------------:|:---:|:---:|:---:|:---:|:---:|:---:|:---:|
| **Zárvatartási idő** | $0$ | $4$ | $0$ | $3$ | $2$ | $0$ | $0$ |


Megjegyzés: az összes zárvatartási idő összege $9$, ami nem több, mint a $K = 10$. 
A $0$., az $1$. és a $3$. városok elérhetők  az $X$ városból ($X = 0$), míg az $1$., a $2$. és a $4$. városok elérhetők az $Y$ városból ($Y = 2$).
Így a kényelmi pontszám $3 + 3 = 6$.
Nincs olyan zárvatartási idő hozzárendelés, amelynek kényelmi pontszáma nagyobb lenne $6$-nál, ezért a függvénynek $6$-t kell visszaadnia.

Tekintsük a következő hívást:

```
max_score(4, 0, 3, 20, [0, 1, 2], [1, 2, 3], [18, 1, 19])
```

Ez az alábbi úthálózatot írja le:

![](closing-sample2.png)

Tegyük fel, hogy a zárvatartási időket a következőképpen osztják ki:

| **Város**         | $0$ | $1$ | $2$ | $3$ |
|:----------------:|:---:|:---:|:---:|:---:|
| **Zárvatartási idő** | $0$ | $1$ | $19$| $0$ |

A $0$. város elérhető az $X$ ($X = 0$) városból, míg a $2$. és a  $3$. elérhető az $Y$. ($Y = 3$) városból.
Így a kényelmi pontszám $1 + 2 = 3$.
Nincs olyan zárvatartási idő hozzárendelés, amelynek kényelmi pontszáma nagyobb lenne $3$-nál, ezért a függvénynek $3$-t kell visszaadnia.


## Feltételek

* $2 \le N \le 200\,000$
* $0 \le X \lt Y \lt N$
* $0 \le K \le 10^{18}$
* $0 \le U[j] \lt V[j] \lt N$ (minden $j$-re, ahol $0 \le j \le N - 2$)
* $1 \le W[j] \le 10^6$ (minden $j$-re, ahol  $0 \le j \le N - 2$)
* Bármely városból bármely másik városba el lehet utazni az utak használatával.
* $S_N \le 200\,000$, ahol $S_N$ az $N$-ek összege az összes `max_score` hívásban.


## Részfeladatok

Azt mondjuk, hogy egy úthálózat **lineáris**, ha az $i$. út az $i$. és az $i + 1$. városokat köti össze (minden $i$ esetében, ahol $0 \le i \le N - 2$).

1. (8 pont) Az útvonal hossza az $X$. és az $Y$. város közt nagyobb, mint  $2K$.
1. (9 pont) $S_N \le 50$, az úthálózat lineáris.
1. (12 pont) $S_N \le 500$, az úthálózat lineáris.
1. (14 pont) $S_N \le 3\,000$, az úthálózat lineáris.
1. (9 pont) $S_N \le 20$
1. (11 pont) $S_N \le 100$
1. (10 pont) $S_N \le 500$
1. (10 pont) $S_N \le 3\,000$
1. (17 pont) Nincs további megkötés.


## Mintaértékelő

Jelölje $C$ a tesztesetek számát, azaz a `max_score` hívások számát.
A mintaértékelő a következő formátumban olvassa be a bemenetet:

* $1$. sor: $C$

A $C$ tesztesetek leírása az alábbi.

A mintaértékelő az egyes tesztesetek leírását a következő formátumban olvassa be:

* $1$. sor: $N \; X \; Y \; K$
* $2 + j$. sor (ahol $0 \le j \le N - 2$): $U[j] \; V[j] \; W[j]$

A mintaértékelő minden egyes forgatókönyvhöz egyetlen sort nyomtat, a következő formátumban:

* $1$. sor: a `max_score` visszatérési értéke
