# זמן סגירה

הונגריה היא מדינה עם $N$ ערים, הממוספרות מ-$0$ עד $N - 1$.

הערים מחוברות באמצעות $N - 1$ כבישים *דו-סטריים*, הממוספרים מ-$0$ עד $N - 2$.
לכל $j$ המקיים $0 \le j \le N - 2$, הכביש $j$ מחבר בין העיר $U[j]$ והעיר $V[j]$ והוא באורך $W[j]$,
כלומר, הוא מאפשר לטייל בין הערים ב-$W[j]$ יחידות זמן.
כל כביש מחבר בין שתי ערים שונות, וכל זוג ערים מחובר על ידי כביש אחד לכל היותר.

**מסלול** בין שתי ערים שונות $a$ ו-$b$ הוא רצף ערים שונות $p_0, p_1, \ldots, p_t$, כך ש:
 * $p_0 = a$,
 * $p_t = b$,
 * לכל $i$ ($0 \le i \lt t$), ישנו כביש המחבר בין הערים $p_i$ ו-$p_{i + 1}$.

ניתן לטייל מכל עיר לכל עיר אחרת באמצעות הכבישים, כלומר, קיים מסלול בין כל שתי ערים שונות.
ניתן להראות שמסלול זה הוא ייחודי לכל זוג ערים שונות.

ה**אורך** של מסלול $p_0, p_1, \ldots, p_t$ הוא סכום האורכים של $t$ הכבישים המחברים ערים עוקבות לאורך המסלול.

בהונגריה, אנשים רבים משתתפים בחגיגות יום ההקמה הלאומי בחלק מהערים הגדולות. כשהחגיגות מסתיימות, הם חוזרים לבתים שלהם. הממשלה רוצה למנוע מהקהל להפריע למקומיים, אז היא מתכננת לסגור את הערים בזמנים מסוימים.
לכל עיר מוקצה  **זמן סגירה** אי-שלילי על ידי הממשלה.
הממשלה החליטה שאסור שסכום כל זמני הסגירה יהיה מעל $K$.
ליתר דיוק, לכל $i$ בין $0$ ל-$N - 1$, כולל, זמן הסגירה המוקצה לעיר $i$ הוא מספר שלם אי-שלילי $c[i]$.
אסור שהסכום של כל ה-$c[i]$ יהיה גדול מ-$K$.

הביטו בעיר $a$ והקצאה כלשהי של זמני סגירה.
נאמר שהעיר $b$ **נגישה** מהעיר $a$ אם ורק אם או $b = a$, או המסלול $p_0, \ldots, p_t$ בין שתי הערים הללו (כלומר בפרט $p_0 = a$ ו-$p_t = b$) מקיים את התנאים הבאים:
 * אורך המסלול $p_0, p_1$ הוא לכל היותר $c[p_1]$, וגם
 * אורך המסלול $p_0, p_1, p_2$ הוא לכל היותר $c[p_2]$, וגם
 * $\ldots$
 * אורך המסלול $p_0, p_1, p_2, \ldots, p_t$ הוא לכל היותר $c[p_t]$.

בשנה זו, שני אתרי הפסטיבל העיקריים ממוקמים בעיר $X$ ובעיר $Y$.
עבור כל הקצאה של זמני סגירה, **ערך הנוחות** מוגדר כסכום שני המספרים הבאים:
- מספר הערים הנגישות מהעיר $X$.
- מספר הערים הנגישות מהעיר $Y$.

שימו לב שאם עיר כלשהי נגישה מהעיר $X$ וגם נגישה מהעיר $Y$, היא נספרת *פעמיים* לערך הנוחות.

משימתכם היא לחשב את ערך הנוחות המקסימלי שניתן להשיג באמצעות הקצאה כלשהי של זמני סגירה.


## פרטי מימוש

עליכם לממש את הפונקציה הבאה.

```
int max_score(int N, int X, int Y, int64 K, int[] U, int[] V, int[] W)
```

* $N$: מספר הערים.
* $X$, $Y$: הערים עם אתרי הפסטיבל המרכזיים.
* $K$: החסם העליון על סכום זמני הסגירה.
* $U$, $V$: מערכים באורך $N - 1$ המתארים את חיבורי הכבישים.
* $W$: מערך באורך $N - 1$ המתאר את אורכי הכבישים.
* על פונקציה זו להחזיר את ערך הנוחות המקסימלי שניתן להשיג על ידי הקצאה כלשהי של זמני סגירה.
* פונקציה זו יכולה להקרא **מספר פעמים** בכל טסטקייס.


## דוגמה

התבוננו בקריאה הבאה:

```
max_score(7, 0, 2, 10,
          [0, 0, 1, 2, 2, 5], [1, 3, 2, 4, 5, 6], [2, 3, 4, 2, 5, 3])
```

היא מתאימה למערכת הכבישים הבאה:

![](closing-sample1.png)

הניחו שזמני הסגירה מוקצים כדלקמן:

| $6$ | $5$ | $4$ | $3$ | $2$ | $1$ | $0$ | **עיר**         |
|:----------------:|:---:|:---:|:---:|:---:|:---:|:---:|:---:|
| $0$ | $0$ | $2$ | $3$ | $0$ | $4$ | $0$ | **זמן סגירה** |

שימו לב שהסכום של כל זמני הסגירה הוא $9$, שזה לא יותר מ-$K = 10$. 
הערים $0$, $1$, ו-$3$ נגישות מהעיר $X$ ($X = 0$), בזמן שהערים $1$, $2$, ו-$4$ נגישור מהעיר $Y$ ($Y = 2$).
לכן, ערך הנוחות הוא $3 + 3 = 6$.
לא קיימת הקצאה של זמני סגירה עם ערך נוחות גדול מ-$6$, אז על הפונקציה להחזיר $6$.

התבוננו גם בקריאה הבאה:

```
max_score(4, 0, 3, 20, [0, 1, 2], [1, 2, 3], [18, 1, 19])
```

היא מתאימה למערכת הכבישים הבאה:

![](closing-sample2.png)

הניחו שזמני הסגירה מוקצים כדלקמן:

| $3$ | $2$ | $1$ | $0$ |  **עיר**         |
|:----------------:|:---:|:---:|:---:|:---:|
| $0$ | $19$ | $1$| $0$ | **זמן סגירה** |

העיר $0$ נגישה מהעיר $X$ ($X = 0$), בזמן שהערים $2$ ו-$3$ נגישות מהעיר $Y$ ($Y = 3$).
לכן, ערך הנוחות הוא $1 + 2 = 3$.
לא קיימת הקצאה של זמני סגירה עם ערך נוחות גדול מ- $3$, אז על הפונקציה להחזיר $3$.


## מגבלות

* $2 \le N \le 200\,000$
* $0 \le X \lt Y \lt N$
* $0 \le K \le 10^{18}$
* $0 \le U[j] \lt V[j] \lt N$ (לכל $j$ המקיים $0 \le j \le N - 2$)
* $1 \le W[j] \le 10^6$ (לכל $j$ המקיים $0 \le j \le N - 2$)
* ניתן לטייל מכל עיר לכל עיר אחרת באמצעות הכבישים.
* $S_N \le 200\,000$, כש-$S_N$ הוא הסכום של $N$ על פני כל הקריאות ל-`max_score` בכל טסטקייס.


## תתי משימות

נאמר שמערכת כבישים היא **לינארית** אם הכביש $i$ מחבר בין הערים $i$ ו-$i + 1$ (לכל $i$ המקיים $0 \le i \le N - 2$).

1. (8 נקודות) אורך המסלול מהעיר $X$ לעיר $Y$ גדול מ-$2K$.
1. (9 נקודות) $S_N \le 50$, מערכת הכבישים היא לינארית.
1. (12 נקודות) $S_N \le 500$, מערכת הכבישים היא לינארית.
1. (14 נקודות) $S_N \le 3\,000$, מערכת הכבישים היא לינארית.
1. (9 נקודות) $S_N \le 20$
1. (11 נקודות) $S_N \le 100$
1. (10 נקודות) $S_N \le 500$
1. (10 נקודות) $S_N \le 3\,000$
1. (17 נקודות) ללא מגבלות נוספות.


## גריידר לדוגמה

נסמן ב-$C$ את מספר התרחישים, כלומר, מספר הקריאות ל-`max_score`.
הגריידר לדוגמה קורא את הקלט בפורמט הבא:

* שורה $1$: $C$

התיאורים של $C$ תרחישים לאחר מכן.

הגריידר לדוגמה קורא את התיאור של כל תרחיש בפורמט הבא:

* שורה $1$: $N \; X \; Y \; K$
* שורה $2 + j$ ($0 \le j \le N - 2$): $U[j] \; V[j] \; W[j]$

הגריידר לדוגמה מדפיס שורה אחת עבור כל תרחיש, בפורמט הבא:

* שורה $1$: ערך החזרה של `max_score`




