# Uždarymo laikas

Vengrija yra šalis su $N$ miestų, sunumeruotų nuo $0$ iki $N - 1$.

Miestus jungia $N - 1$ *dvikrypčių* kelių, sunumeruotų nuo $0$ iki $N - 2$.
Kiekvienam $j$ ($0 \le j \le N - 2$), kelias $j$ jungia miestą $U[j]$ su miestu $V[j]$ ir jo ilgis yra $W[j]$.
Kitaip tariant, iš vieno miesto į kitą galima nukeliauti per $W[j]$ laiko vienetų.
Kiekvienas kelias jungia du skirtingus miestus, ir kiekvieną miestų porą jungia daugiausiai vienas kelias.

**Maršrutą** tarp dviejų skirtingų miestų $a$ ir $b$ sudaro skirtingų miestų seka $p_0, p_1, \ldots, p_t$, kuriai galioja:
 * $p_0 = a$,
 * $p_t = b$,
 * kiekvienam $i$ ($0 \le i \lt t$) egzistuoja kelias, jungiantis $p_i$ ir $p_{i + 1}$.

Įmanoma nukeliauti iš bet kurio vieno miesto į bet kurį kitą miestą. Kitaip sakant, egzistuoja maršrutas tarp bet kurių dviejų skirtingų miestų.
Galima įrodyti, kad maršrutas tarp bet kurių dviejų miestų yra vienintelis.

Maršruto $p_0, p_1, \ldots, p_t$ **ilgis** lygus $t$ kelių, jungiančių gretimus miestus, ilgių sumai.

Vengrijoje daug žmonių dalyvauja Nepriklausomybės dienos iškilmėse dviejuose didžiuosiuose miestuose.
Pasibaigus šventei jie nori sugrįžti į savo namus.
Vyriausybė nori, kad minia nesutrikdytų vietinių gyventojų, taigi jie planuoja tam tikrais laikais uždaryti visus miestus.
Kiekvienam miestui vyriausybė priskirs neneigiamą **uždarymo laiką**.
Vyriausybė nusprendė, kad visų uždarymo laikų suma negali būti didesnė, nei $K$.
Tai yra, kiekvienam $i$ nuo $0$ iki $N - 1$ imtinai, $i$-ojo miesto uždarymo laikas yra neneigiamas sveikasis skaičius $c[i]$.
Visų $c[i]$ suma negali viršyti $K$.

Panagrinėkime miestą $a$ ir tam tikrą uždarymo laikų priskyrimą.
Miestą $b$ vadinsime **pasiekiamu** iš miesto $a$ tada ir tik tada, jei $b = a$, arba jei maršrutas $p_0, \ldots, p_t$ tarp šių dviejų miestų (čia $p_0 = a$ ir $p_t = b$) tenkina šiuos reikalavimus:
 * maršruto $p_0, p_1$ ilgis yra daugiausiai $c[p_1]$, ir
 * maršruto $p_0, p_1, p_2$ ilgis yra daugiausiai $c[p_2]$, ir
 * $\ldots$
 * maršruto $p_0, p_1, p_2, \ldots, p_t$ ilgis yra daugiausiai $c[p_t]$.

Šiemet du pagrindiniai miestai, kuriuose rengiamas festivalis, yra miestai $X$ ir $Y$.
Kiekvienam miestų uždarymo priskyrimui apibrėžiamas to priskyrimo **patogumas**, kuris lygus šių dviejų skaičių sumai:
- Iš miesto $X$ pasiekiamų miestų skaičius.
- Iš miesto $Y$ pasiekiamų miestų skaičius.

Atkreipkite dėmesį, kad jei miestas pasiekiamas tiek iš miesto $X$, tiek iš miesto $Y$, jis į patogumo vertę įeina *du* kartus.

Suskaičiuokite didžiausią patogumą, kurį galima gauti priskiriant uždarymo laikus.


## Realizacija

Jums reikia parašyti šią funkciją.

```
int max_score(int N, int X, int Y, int64 K, int[] U, int[] V, int[] W)
```

* $N$: miestų skaičius.
* $X$, $Y$: pagrindiniai miestai, kuriuose vyks festivalis.
* $K$: maksimali uždarymo laikų sumos vertė.
* $U$, $V$: $N - 1$ ilgio masyvai.
* $W$: $N - 1$ ilgio masyvas, apibūdinantis kelių ilgius.
* Ši funkcija turėtų grąžinti didžiausią patogumą, kurį galimą pasiekti miestams priskiriant uždarymo laikus.
* Ši funkcija gali būti iškviesta **kelis kartus** kiekvienam testui.


## Pavyzdys

Panagrinėkime šį iškvietimą:

```
max_score(7, 0, 2, 10,
          [0, 0, 1, 2, 2, 5], [1, 3, 2, 4, 5, 6], [2, 3, 4, 2, 5, 3])
```

Tai atitinka šį kelių tinklą:

![](closing-sample1.png)

Tarkime, kad uždarymų laikus priskiriame taip:

| **Miestas**         | $0$ | $1$ | $2$ | $3$ | $4$ | $5$ | $6$ |
|:----------------:|:---:|:---:|:---:|:---:|:---:|:---:|:---:|
| **Uždarymo laikas** | $0$ | $4$ | $0$ | $3$ | $2$ | $0$ | $0$ |

Atkreipkite dėmesį, kad visų uždarymo laikų suma yra $9$, kuri nėra didesnė už $K = 10$. 
Miesai $0$, $1$ ir $3$ yra pasiekiami iš miesto $X$ ($X = 0$), o miestai $1$, $2$, ir $4$ yra pasiekiami iš miesto $Y$ ($Y = 2$).
Taigi patogumo vertė yra $3 + 3 = 6$.
Nėra tokio uždarymo laikų priskyrimo kurio patogumo vertė didesnė už $6$, taigi funkcija turėtų grąžinti $6$.

Taip pat panagrinėkime šį funkcijos iškvietimą:

```
max_score(4, 0, 3, 20, [0, 1, 2], [1, 2, 3], [18, 1, 19])
```

Tai atitinka šį kelių tinklą:

![](closing-sample2.png)

Tarkime, kad uždarymų laikus priskiriame taip:

| **Miestas**         | $0$ | $1$ | $2$ | $3$ |
|:----------------:|:---:|:---:|:---:|:---:|
| **Uždarymo laikas** | $0$ | $1$ | $19$| $0$ |

Miestas $0$ yra pasiekiamas iš miesto $X$ ($X = 0$), o miestai $2$ ir $3$ yra pasiekiami iš miesto $Y$ ($Y = 3$).
Taigi patogumo vertė yra $1 + 2 = 3$.
Nėra tokio uždarymo laikų priskyrimo, kurio patogumo vertė būtų didesnė už $3$, taigi funkcija turėtų grąžinti $3$.


## Ribojimai

* $2 \le N \le 200\,000$
* $0 \le X \lt Y \lt N$
* $0 \le K \le 10^{18}$
* $0 \le U[j] \lt V[j] \lt N$ (kiekvienam $j$, kuriam $0 \le j \le N - 2$)
* $1 \le W[j] \le 10^6$ (kiekvienam $j$, kuriam $0 \le j \le N - 2$)
* Įmanoma nukeliauti iš bet kurio miesto į bet kurį kitą miestą.
* $S_N \le 200\,000$, kur $S_N$ yra $N$ verčių suma visiems `max_score` iškvietimams viename teste.


## Dalinės užduotys

Sakome, kad kelių tinklas yra **tiesinis**, jei kelias $i$ jungia miestus $i$ ir $i + 1$ (kiekvienam $i$, kur $0 \le i \le N - 2$).

1. (8 taškai) Maršruto ilgis tarp miestų $X$ ir $Y$ yra didesnis už $2K$.
1. (9 taškai) $S_N \le 50$, kelių tinklas yra tiesinis.
1. (12 taškų) $S_N \le 500$, kelių tinklas yra tiesinis.
1. (14 taškų) $S_N \le 3\,000$, kelių tinklas yra tiesinis.
1. (9 taškai) $S_N \le 20$
1. (11 taškų) $S_N \le 100$
1. (10 taškų) $S_N \le 500$
1. (10 taškų) $S_N \le 3\,000$
1. (17 taškų) Papildomų ribojimų nėra.


## Pavyzdinė vertinimo programa

Pažymėkime $C$ scenarijų skaičių (t. y. `max_score` iškvietimų skaičių).
Pavyzdinė vertinimo sistema skaito duomenis šiuo formatu:

* $1$-a eilutė: $C$

Toliau pateikiama $C$ scenarijų.

Pavyzdinė vertinimo programa skaito kiekvieną scenarijų pateiktą šiuo formatu:

* $1$-a eilutė: $N \; X \; Y \; K$
* $(2 + j)$-a eilutė ($0 \le j \le N - 2$): $U[j] \; V[j] \; W[j]$

Pavyzdinė vertinimo programa išveda vieną eilutę kiekvienam scenarijui šiuo formatu:

* $1$-oji eilutė: `max_score` funkcijos grąžinta vertė.
