# 關閉時刻

匈牙利是一個有 $N$ 個城市的國家，城市編號從 $0$ 到 $N - 1$。

這些城市由 $N - 1$ 條 *雙向* 道路連接，編號從 $0$ 到 $N - 2$。
對於每個 $j$，滿足 $0 \le j \le N - 2$，道路 $j$ 連接城市 $U[j]$ 和城市 $V[j]$，並且長度為 $W[j]$，
也就是說，可以在 $W[j]$ 個單位時間內從一個城市到達另一個城市。
每條道路連接兩個不同的城市，每對城市之間最多只有一條道路。

兩個不同城市 $a$ 和 $b$ 之間的一條 **路徑** 是一個序列 $p_0, p_1, \ldots, p_t$，其中 
 * $p_0 = a$
 * $p_t = b$，
 * 對於每個 $i$（$0 \le i \lt t$），城市 $p_i$ 和城市 $p_{i + 1}$ 之間有一條道路。

可以通過使用這些道路從任何城市到達任何其他城市，也就是說，每兩個不同城市之間都存在一條路徑。
所此可見，對於每對不同的城市，這條路徑是唯一的。

一條路徑 $p_0, p_1, \ldots, p_t$ 的 **長度** 是沿著路徑連接相鄰城市的 $t$ 條道路的總長度的總和。

在匈牙利，許多人前往參加在兩個主要城市舉行的基金會日慶祝活動。
慶祝活動結束後，他們將會回家。
政府希望防止人群打擾當地居民，因此他們計劃所有的城市都會在一些特定時刻被關閉。
政府將為每個城市分配一個非負的 **關閉時刻**。
政府已經決定所有關閉時刻的總和不能超過 $K$。
更具體地說，對於每個 $i$（$0$ 到 $N - 1$ 之間的值），分配給城市 $i$ 的關閉時刻是非負整數 $c[i]$。
所有 $c[i]$ 的總和不能大於 $K$。

考慮一個城市 $a$ 和一些關閉時刻的分配。
我們說城市 $b$ 從城市 $a$ 是**可達**的，當且僅當以下條件滿足： $b = a$，或者這兩個城市之間的路徑 $p_0, \ldots, p_t$（$p_0 = a$ 和 $p_t = b$）並滿足以下條件：
   - 路徑 $p_0, p_1$ 的長度最多為 $c[p_1]$，且
   - 路徑 $p_0, p_1, p_2$ 的長度最多為 $c[p_2]$，且
   - $\ldots$
   - 路徑 $p_0, p_1, p_2, \ldots, p_t$ 的長度最多為 $c[p_t]$。

今年，兩個主要的慶祝活動場地位於城市 $X$ 和城市 $Y$。
對於每個關閉時刻的分配，**便利分數** 定義為以下兩個數字的和：
- 從城市 $X$ 可達的城市數量。
- 從城市 $Y$ 可達的城市數量。

注意，如果一個城市從城市 $X$ 可達且從城市 $Y$ 可達，則它對便利分數的計算會被用 *兩次*。

你的任務是計算通過某些關閉時刻的分配可以達到的最大便利分數。


## 實現細節

你應該實現以下子程式。

```
int max_score(int N, int X, int Y, int64 K, int[] U, int[] V, int[] W)
```

* $N$：城市的數量。
* $X$、$Y$：主要慶祝活動場地的城市。
* $K$：關閉時刻總和的上限。
* $U$、$V$：長度為 $N - 1$ 的陣列，描述道路連接。
* $W$：長度為 $N - 1$ 的陣列，描述道路長度。
* 這個過程應該返回通過某些關閉時刻的分配可以達到的最大便利分數。
* 在每個測試案例中，這個過程可能會被 **多次** 調用。
## 範例

考慮以下的調用：

```
max_score(7, 0, 2, 10,
          [0, 0, 1, 2, 2, 5], [1, 3, 2, 4, 5, 6], [2, 3, 4, 2, 5, 3])
```

這對應到以下的道路網絡：

![](closing-sample1.png)

假設關閉時刻分配如下：

| **城市**         | $0$ | $1$ | $2$ | $3$ | $4$ | $5$ | $6$ |
|:----------------:|:---:|:---:|:---:|:---:|:---:|:---:|:---:|
| **關閉時刻** | $0$ | $4$ | $0$ | $3$ | $2$ | $0$ | $0$ |

注意到所有關閉時刻的總和為 $9$，不超過 $K = 10$。
城市 $0$、$1$ 和 $3$ 可以從城市 $X$ ($X = 0$) 到達，而城市 $1$、$2$ 和 $4$ 可以從城市 $Y$ ($Y = 2$) 到達。
因此，方便度分數為 $3 + 3 = 6$。
沒有任何關閉時刻分配的方便度分數超過 $6$，所以該程序應該返回 $6$。

再考慮以下的調用：

```
max_score(4, 0, 3, 20, [0, 1, 2], [1, 2, 3], [18, 1, 19])
```

這對應到以下的道路網絡：

![](closing-sample2.png)

假設關閉時刻分配如下：

| **城市**         | $0$ | $1$ | $2$ | $3$ |
|:----------------:|:---:|:---:|:---:|:---:|
| **關閉時刻** | $0$ | $1$ | $19$| $0$ |

城市 $0$ 可以從城市 $X$ ($X = 0$) 到達，而城市 $2$ 和 $3$ 可以從城市 $Y$ ($Y = 3$) 到達。
因此，方便度分數為 $1 + 2 = 3$。
沒有任何關閉時刻分配的方便度分數超過 $3$，所以該程序應該返回 $3$。


## 約束條件

* $2 \le N \le 200\,000$
* $0 \le X \lt Y \lt N$
* $0 \le K \le 10^{18}$
* $0 \le U[j] \lt V[j] \lt N$（對於每個 $j$，滿足 $0 \le j \le N - 2$）
* $1 \le W[j] \le 10^6$（對於每個 $j$，滿足 $0 \le j \le N - 2$）
* 可以通過道路從任何城市到達任何其他城市。
* $S_N \le 200\,000$，其中 $S_N$ 是在每個測試數據內的所有調用  `max_score` 時的 $N$ 的總和。


## 子任務

我們稱道路網絡為**線性**，如果道路 $i$ 連接城市 $i$ 和 $i + 1$（對於每個 $i$，滿足 $0 \le i \le N - 2$）。

1. （8 分）從城市 $X$ 到城市 $Y$ 的路徑長度大於 $2K$。
1. （9 分）$S_N \le 50$，道路網絡是線性的。
1. （12 分）$S_N \le 500$，道路網絡是線性的。
1. （14 分）$S_N \le 3\,000$，道路網絡是線性的。
1. （9 分）$S_N \le 20$
1. （11 分）$S_N \le 100$
1. （10 分）$S_N \le 500$
1. （10 分）$S_N \le 3\,000$
1. （17 分）沒有額外的約束條件。


## 樣例評試程式

令 $C$ 表示情境的數量，即對 `max_score` 的調用次數。
樣例評試程式以以下格式讀取輸入：

* 第 $1$ 行：$C$

接下來是 $C$ 個情境的描述。

樣例評試程式以以下格式讀取每個情境的描述：

* 第 $1$ 行：$N \; X \; Y \; K$
* 第 $2 + j$ 行（$0 \le j \le N - 2$）：$U[j] \; V[j] \; W[j]$

樣例評試程式對於每個情境都會輸出一行，以以下格式：

* 第 $1$ 行：`max_score` 的返回值

