# Затварање

Србија је држава са $N$ села (топонима), нумерисаних од $0$ до $N - 1$.

Села су повезана са $N - 1$ *двосмерних* аутопутева, нумерисаним од $0$ до $N - 2$.
За свако $j$ за које важи $0 \le j \le N - 2$, аутопут $j$ повезује село $U[j]$ и село $V[j]$ и има дужину $W[j]$,
то јесте, дозвољава да се отпутује између села у $W[j]$ јединица времена.
Сваки аутопут повезује два различита села, и сваки пар села је повезан највише једним аутопутем.

**Пут** између два различита села $a$ и $b$ је низ $p_0, p_1, \ldots, p_t$ различитих села, тако да је:
 * $p_0 = a$,
 * $p_t = b$,
 * за свако $i$ ($0 \le i \lt t$), постоји аутопут који повезује $p_i$ и $p_{i + 1}$.

Могуће је отпутовати од било ког села до било ког другог села користећи аутопутеве, то јесте, постоји пут измежу свака два различита града.
Приметимо да је овај пут јединствен за сваки пар различитих градова.

**Дужина** пута $p_0, p_1, \ldots, p_t$ је збир дужина $t$ аутопутева који повезују узастопне градове на путу.

У Србији, многи људи путују да посете разне фестивале хране који се традиционално одржавају у неким од села после Васкрса.
Када се све прославе заврше, они се враћају својим кућама.
Министарство за туризам и провод жели да спречи хорде људи да узнемиравају локално становништво, тако да планирају да закључају градове у одређено време.
Сваком граду ће бити додељено ненегативно **време затварања** од стране министарства.
Министарство је одлучило да збир свих времена затварања не сме бити више од $K$.
Прецизније, за свако $i$ од $0$ до $N - 1$, време затварања додељено селу $i$ је ненегативан цео број $c[i]$.
Збир свих вредности $c[i]$ не сме бити већи од $K$.

Уочимо неко село $a$ и неку расподелу времена затварања свих села.
Кажемо да је град $b$ is **могуће посетити** из села $a$ ако и само ако или $b = a$, или пут $p_0, \ldots, p_t$ између та два села (дакле $p_0 = a$ и $p_t = b$) задовољава следеће услове:
 * дужина пута $p_0, p_1$ је највише $c[p_1]$, и
 * дужина пута $p_0, p_1, p_2$ је највише $c[p_2]$, и
 * $\ldots$
 * дужина пута $p_0, p_1, p_2, \ldots, p_t$ је највише $c[p_t]$.

Ове године, две главне прославе се одржавају у селима $X$ и $Y$.
За сваку расподелу времена затварања, **угодност** је дефинисана као збир следећа два броја:
- Број села кој је могуће посетити из села $X$.
- Број села кој је могуће посетити из села $Y$.

Уколико је неко село могуће посетити и из села $X$ и из села $Y$, оно се рачуна *двапут* у рачунању угодности.

Ваш задатак је да израчунате максималну угодност која се може остварити неком расподелом времена затварања.

## Детаљи имплементације

Треба да имплементирате следећу процедуру.

```
int max_score(int N, int X, int Y, int64 K, int[] U, int[] V, int[] W)
```

* $N$: број села.
* $X$, $Y$: села у којима се одржавају главне прославе.
* $K$: максимална дозвољена сума времена затварања.
* $U$, $V$: низови дужине $N - 1$ који описују аутопутеве.
* $W$: низ дужине $N - 1$ који описује дужине аутопутева.
* Процеура треба да врати највећу угодности која може да се достигне неком расподелом времена затварања.
* Процедура може бити позвана **више пута** у сваком тест примеру.


## Пример

Посматрајмо следећи позив функције:

```
max_score(7, 0, 2, 10,
          [0, 0, 1, 2, 2, 5], [1, 3, 2, 4, 5, 6], [2, 3, 4, 2, 5, 3])
```

Ово одговара следећој мрежи аутопутева:

![](closing-sample1.png)

Претпоставимо да су времена затварања следећа:

| **Село**         | $0$ | $1$ | $2$ | $3$ | $4$ | $5$ | $6$ |
|:----------------:|:---:|:---:|:---:|:---:|:---:|:---:|:---:|
| **Време затварања** | $0$ | $4$ | $0$ | $3$ | $2$ | $0$ | $0$ |

Приметимо да је збир свих времена затварања $9$, што није више од $K = 10$. 
Села $0$, $1$, и $3$ је могуће посетити из села $X$ ($X = 0$), док је села $1$, $2$, и $4$ могуће посетити из села $Y$ ($Y = 2$).
Дакле, угодност је $3 + 3 = 6$.
Не постоји расподела времена затварања у којој је угодност већа од  $6$, тако да процедура треба да врати $6$.

Такође посматрајмо следећи позив:

```
max_score(4, 0, 3, 20, [0, 1, 2], [1, 2, 3], [18, 1, 19])
```

Ово одговара следећој мрежи аутопутева:

![](closing-sample2.png)

Претпоставимо да су времена затварања следећа:

| **Град**         | $0$ | $1$ | $2$ | $3$ |
|:----------------:|:---:|:---:|:---:|:---:|
| **Време затварања** | $0$ | $1$ | $19$| $0$ |

Село $0$ је могуће посетити из села $X$ ($X = 0$), док је села $2$ и $3$ могуће посетити из села $Y$ ($Y = 3$).
Дакле, угодност је $1 + 2 = 3$.
Не постоји расподела времена затварања у којој је угодност већа од  $3$, тако да процедура треба да врати $3$.


## Ограничења

* $2 \le N \le 200\,000$
* $0 \le X \lt Y \lt N$
* $0 \le K \le 10^{18}$
* $0 \le U[j] \lt V[j] \lt N$ (за свако $j$ тако да је $0 \le j \le N - 2$)
* $1 \le W[j] \le 10^6$ (за свако $j$ тако да је $0 \le j \le N - 2$)
* Могуће је отпутовати од било ког до било ког другог села користећи аутопутеве.
* $S_N \le 200\,000$, где је $S_N$ збир $N$-ова по свим позивима процедуре `max_score`.


## Подзадаци

Кажемо да је мрежа аутопутева **линијска** ако аутопут $i$ повезује села $i$ и $i + 1$ (за свако $i$ тако да је $0 \le i \le N - 2$).

1. (8 поена) Дужина пута од села $X$ до села $Y$ је већа од $2K$.
1. (9 поена) $S_N \le 50$, мрежа путева је линијска.
1. (12 поена) $S_N \le 500$, мрежа путева је линијска.
1. (14 поена) $S_N \le 3\,000$, мрежа путева је линијска.
1. (9 поена) $S_N \le 20$
1. (11 поена) $S_N \le 100$
1. (10 поена) $S_N \le 500$
1. (10 поена) $S_N \le 3\,000$
1. (17 поена) Нема додатних ограничења.


## Пример оцењивача (sample grader)

Нека $C$ означава број сценарија, то јесте, број позива функције `max_score`.
Оцењивач учитава улаз у следећем формату:

* линија $1$: $C$

Описи $C$ сценарија се налазе испод.

Оцењивач учитава опис сваког сценарија у следећем формату:

* линија $1$: $N \; X \; Y \; K$
* линија $2 + j$ ($0 \le j \le N - 2$): $U[j] \; V[j] \; W[j]$

Оцењивач исписује једну линију по сценарију, у следећем формату:

* линија $1$: повратна вредност функције `max_score`
