# Čas zatvorenia

Maďarsko je krajina majúca $N$ miest, očíslovaných od $0$ do $N - 1$.

Mestá sú spojené $N - 1$ *obojsmernými* cestami, očíslovanými od  $0$ do $N - 2$. Pre každé $j$  také, že $0 \le j \le N - 2$, cesta $j$ spája mestá $U[j]$ a $V[j]$ a má dĺžku $W[j]$, teda umožňuje cestovať medzi týmito dvoma mestami za $W[j]$ časových jednotiek.
Každá cesta spája dve rôzne mestá a každá dvojica miest je spojená najviac jednou cestou.

**Spojenie** medzi dvoma rôznymi mestami $a$ a $b$ je postupnosť $p_0, p_1, \ldots, p_t$ navzájom rôznych miest, pre ktorú platí:
 * $p_0 = a$,
 * $p_t = b$,
 * pre každé $i$ ($0 \le i \lt t$) existuje cesta spájajúca mestá $p_i$ a $p_{i + 1}$.

Z každého mesta sa dá dostať do každého použitím ciest, t.j. pre každú dvojicu miest existuje spojenie medzi nimi. Dá sa dokázať, že pre každú dvojicu rôznych miest existuje práve jedno spojenie medzi nimi.

 **Dĺžka** spojenia $p_0, p_1, \ldots, p_t$ je súčtom dĺžok jednotlivých $t$ ciest, ktorými toto spojenie vedie.

Množstvo ľudí v Maďarsku cestuje na oslavy Sviatku ústavy do dvoch významných miest. Po skončení osláv sa vracajú domov. Vláda chce zabrániť rušeniu pokoja miestnych davmi, takže naplánovala postupne zatvoriť všetky mestá.
Každému mestu bude vládou priradené nezáporné číslo vyjadrujúce  **čas zatvorenia**.
Vláda sa rozhodla, že celkový súčet časov zatvorenia neprekročí $K$. Presnejšie, pre každé $i$ medzi $0$ a $N - 1$ (vrátane), mestu $i$ bude priradený čas zatvorenia: nezáporné celé číslo $c[i]$. Súčet všetkých časov zatvorenia $c[i]$ nesmie prekročiť $K$.

Pozrime sa na mesto $a$ a nejaké priradenie časov zatvorenia.
Hovoríme, že mesto $b$ je **dosiahnuteľné** z mesta $a$ práve vtedy, keď buď $b = a$ alebo spojenie $p_0, \ldots, p_t$ medzi týmito dvoma mestami (teda $p_0 = a$ a $p_t = b$) spĺňa nasledovné podmienky:
 * dĺžka spojenia $p_0, p_1$ je najviac $c[p_1]$, a zároveň
 * dĺžka spojenia $p_0, p_1, p_2$ je najviac $c[p_2]$, a zároveň
 * $\ldots$
 * dĺžka spojenia $p_0, p_1, p_2, \ldots, p_t$ je najviac $c[p_t]$.

Tento rok sa dve najväčšie oslavy konajú v mestách $X$ a $Y$.
Pre každé priradenie časov zatvorenia je definované **skóre pohodlia** ako súčet dvoch čísel:
- počtu miest dosiahnuteľných z mesta $X$
- počtu miest dosiahnuteľných z mesta $Y$

V prípade, že je mesto dosiahnuteľné aj z mesta $X$ aj z mesta $Y$, je dané mesto zarátané v skóre pohodlia *dvakrát*.

Vašou úlohou je vypočítať maximálne možné skóre pohodlia, ktoré sa dá dosiahnuť korektným priradením časov zatvorenia.


## Implementačné detaily

Vašou úlohou je implementovať nasledujúcu funkciu.

```
int max_score(int N, int X, int Y, int64 K, int[] U, int[] V, int[] W)
```

* $N$: počet miest
* $X$, $Y$: dve mestá s oslavami
* $K$: horné ohraničenie pre súčet časov zatvorenia
* $U$, $V$: polia dĺžky $N - 1$ popisujúce cesty
* $W$: pole dĺžky $N - 1$ udávajúce dĺžky ciest
* Táto funkcia má vrátiť maximálne dosiahnuteľné skóre pohodlia.
* Táto funkcia môže byť v rámci jedného behu programu zavolaná postupne **viackrát**.


## Príklad

Uvažujme nasledovné volanie funkcie:

```
max_score(7, 0, 2, 10,
          [0, 0, 1, 2, 2, 5], [1, 3, 2, 4, 5, 6], [2, 3, 4, 2, 5, 3])
```

Popisuje nasledovnú cestnú sieť:

![](closing-sample1.png)

Pozrime sa na nasledujúce časy zatvorenia:

| **Mesto**         | $0$ | $1$ | $2$ | $3$ | $4$ | $5$ | $6$ |
|:----------------:|:---:|:---:|:---:|:---:|:---:|:---:|:---:|
| **Čas zatvorenia** | $0$ | $4$ | $0$ | $3$ | $2$ | $0$ | $0$ |

Súčet všetkých časov zatvorenia je $9$, teda nie viac ako $K = 10$. 
Z mesta $X$ ($X = 0$) sú dosiahnuteľné mestá $0$, $1$  a $3$. Z mesta $Y$ ($Y = 2$) sú dosiahnuteľné mestá $1$, $2$ a $4$.
Teda skóre pohodlia je $3 + 3 = 6$.
Neexistuje žiadne priradenie časov zatvorenia, ktoré by viedlo k vyššiemu skóre pohodlia ako  $6$, takže funkcia by mala vrátiť hodnotu $6$.

Pre nasledujúce volanie:

```
max_score(4, 0, 3, 20, [0, 1, 2], [1, 2, 3], [18, 1, 19])
```

reprezentujúce cestnú sieť:

![](closing-sample2.png)

Vezmime priradenie časov zatvorenia nasledovne:

| **Mesto**         | $0$ | $1$ | $2$ | $3$ |
|:----------------:|:---:|:---:|:---:|:---:|
| **Čas zatvorenia** | $0$ | $1$ | $19$| $0$ |

Z mesta $X$ ($X = 0$) je dosiahnuteľné len mesto $0$ a z mesta $Y$ ($Y = 3$) sú dosiahnuteľné len mestá $2$ a $3$.
Teda skóre pohodlia je $1 + 2 = 3$.
Neexistuje priradenie časov zatvorenia s vyšším skóre pohodlia ako $3$, takže funkcia má vrátiť $3$.


## Obmedzenia

* $2 \le N \le 200\,000$
* $0 \le X \lt Y \lt N$
* $0 \le K \le 10^{18}$
* $0 \le U[j] \lt V[j] \lt N$ (pre každé $j$ také, že $0 \le j \le N - 2$)
* $1 \le W[j] \le 10^6$ (pre každé $j$ také, že $0 \le j \le N - 2$)
* Z každého mesta je možné dostať sa do každého použitím zadaných ciest.
* V každom teste platí, že $S_N \le 200\,000$. Hodnota $S_N$ je súčet hodnôt $N$ vo všetkých volaniach funkcie `max_score` v danom teste.


## Podúlohy

Cestnú sieť nazývame **lineárna**, ak cesta $i$ spája mestá $i$ a $i + 1$ (pre každé $i$ také, že $0 \le i \le N - 2$).

1. (08 bodov) Dĺžka spojenia medzi mestami $X$ a $Y$ je väčšia ako $2K$.
1. (09 bodov) $S_N \le 50$, pričom cestná sieť je lineárna.
1. (12 bodov) $S_N \le 500$, pričom cestná sieť je lineárna.
1. (14 bodov) $S_N \le 3\,000$, pričom cestná sieť je lineárna.
1. (09 bodov) $S_N \le 20$.
1. (11 bodov) $S_N \le 100$.
1. (10 bodov) $S_N \le 500$.
1. (10 bodov) $S_N \le 3\,000$
1. (17 bodov) Bez ďalších obmedzení


## Ukážkový testovač

Nech $C$ označuje počet scenárov, teda počet volaní fiunkcie `max_score`.
Ukážkový testovač číta vstup v nasledovnom formáte:

* riadok $1$: $C$

Nasleduje popis jednotlivých $C$ scenárov.

Každý scenár je načítaný ukážkovým testovačom vo formáte:

* riadok $1$: $N \; X \; Y \; K$
* riadok $2 + j$ ($0 \le j \le N - 2$): $U[j] \; V[j] \; W[j]$


Ukážkový testovač vypíše pre každý scenár jeden riadok vo formáte:

* riadok $1$: návratová hodnota funkcie `max_score`
